from enum import IntEnum

class {{ enum.class_info.name }}(IntEnum):
    {% for key, value in enum.values.items() %}
    {{ key }} = {{ value }}
    {% endfor %}

    def __str__(self) -> str:
        return str(self.value)
